angular.module('Hopamviet.services', ['ngCordova','Hopamviet.config'])

 .factory('Database',['$cordovaSQLite','DB_CONFIG', '$q', function ($cordovaSQLite, DB_CONFIG, $q) {
		var self = this;
        self.db = null;
		
		self.initDB = function() {
			console.log("initDB");
             self.db = $cordovaSQLite.openDB(DB_CONFIG.name);
		}
		
		self.initDB4WebSQL = function() {
			console.log("initDB for WebSQL");
			self.db = openDatabase(DB_CONFIG.name, "1.0", "DB for Test on Web", 5 * 1024 * 1024);            
		}
		
		self.preloadDataBase = function (enableLog) {
			var deferred = $q.defer();
			if (window.sqlitePlugin === undefined) {
				enableLog && console.log('%c ***************** Starting the creation of the database in the browser ***************** ', 'background: #222; color: #bada55');
				self.db.transaction(function (tx) {
					for (i = 0; i < window.queries.length; i++) {
						enableLog && console.log(window.queries[i]);
						tx.executeSql(queries[i]);
					}
				}, function (error) {
					deferred.reject(error);
				}, function () {
					enableLog && console.log('%c ***************** Completing the creation of the database in the browser ***************** ', 'background: #222; color: #bada55');
					deferred.resolve("OK");
				});
			}
			else {
				deferred.resolve("OK");
			}

			return deferred.promise;
		};

	
		
		self.getDB = function() {
			console.log("getDB");
			return self.db;
		}
		
		self.query = function(query,bindings) {
			//console.log("query = " + query);
			bindings = typeof bindings !=='undefined' ? bindings: [];
			var deferred = $q.defer();
			
			self.db.transaction(function(transaction){
				transaction.executeSql(query, bindings, function(transaction,result){
					deferred.resolve(result);
				},function(transaction,error){
					deferred.reject(error);
				});
			});
			return deferred.promise;
		}
		
		self.fetchAll = function(result) {
			console.log("fetchAll = " + result.rows.length);
			var output = [];			
			
			for(var i = 0; i< result.rows.length; i++){			
				output.push(result.rows.item(i));
			}
			
			return output;
		}
        
        		
		self.fetch = function(result) {
			//console.log("fetch = " + result);
			return result.rows.item(0);
		};			
		
		return self;
        
  }])
	
.factory("DataManager",['Database', '$cordovaSQLite', 'DB_CONFIG' ,function(Database,$cordovaSQLite,DB_CONFIG, $q){	
	var self = this;
	
	self.getSongList = function() {
		var query = "SELECT *  FROM SongTbl";
		return Database.query(query).then(function(result){
			return Database.fetchAll(result);
		});
	}
	
	self.getFavoriteSongList = function() {
		console.log("getFavoriteSongList");
		var query = "SELECT *  FROM SongTbl where song_isfavorite = 1";
		return Database.query(query).then(function(result){
			return Database.fetchAll(result);
		});
	}
	
	self.getSongListByOffset = function(offset) {
		console.log("offset = " + offset);
		var query = "SELECT *  FROM SongTbl order by _id DESC limit ?,?";
		return Database.query(query,[offset,DB_CONFIG.numItem]).then(function(result){
			return Database.fetchAll(result);
		});
	}
    
    self.getLastviewSongListByOffset = function(offset) {
		console.log("offset = " + offset);
		var query = "SELECT *  FROM SongTbl order by song_lastview DESC limit ?,?";
		return Database.query(query,[offset,DB_CONFIG.numItem]).then(function(result){
			return Database.fetchAll(result);
		});
	}
    
    self.getSongNumber = function(){
        var query = "SELECT count(*) as count FROM SongTbl";
        return Database.query(query).then(function(result){
            return Database.fetch(result).count;         
        });	
    }
    
	self.getSongById = function(songId) {
		console.log("songId = " + songId);
		var query = "SELECT *  FROM SongTbl Where song_id = ?";
		return Database.query(query,[songId]).then(function(result){
			return Database.fetch(result);
		});
	}
    
    self.getAuthorBySongId = function(songId){
        console.log("songId = " + songId);
        var query = "select * from ArtistTbl where artist_id IN (select artist_id from Song_Author_Tbl where song_id = ?)";
        return Database.query(query,[songId]).then(function(result){
			return Database.fetch(result);
		});
    }
    
    self.getChordBySongId = function(songId){
        //console.log("songId = " + songId);
        var query = "select * from ChordTbl where chord_id in (select chord_id from Song_Chord_Tbl where song_id = ?)";
        return Database.query(query,[songId]).then(function(result){
            var output = [];
			for(var i = 0; i< result.rows.length; i++){			
				output.push(result.rows.item(i).chord_name);
			}
            
            return output.join(', ');
		});
        
    }
    
    self.getSingerBySongId = function(songId){
        console.log("songId = " + songId);
        var query = "select * from ArtistTbl where artist_id IN (select artist_id from Song_Singer_Tbl where song_id = ?)";
        return Database.query(query,[songId]).then(function(result){
			return Database.fetch(result);
		});
    }
    
    self.getArtistById = function(artistId) {
		console.log("artistId = " + artistId);
		var query = "SELECT *  FROM ArtistTbl Where artist_id = ?";
		return Database.query(query,[artistId]).then(function(result){
			return Database.fetch(result);
		});
	}
	
	self.updateFavoriteSongById = function(isFavorite,songId) {
		console.log("songId = " + songId);
		var query = "UPDATE SongTbl SET song_isfavorite = ? Where song_id = ?";
		return Database.query(query,[isFavorite,songId]).then(function(result){
			console.log("result.rowsAffected = " + result.rowsAffected);
		});
	}
    
    self.updateLastViewSongById = function(songId) {
		console.log("songId = " + songId);
        var lastview = Date.now();
        console.log("lastview = " + lastview);
		var query = "UPDATE SongTbl SET song_lastview = ? Where song_id = ?";
		return Database.query(query,[lastview,songId]).then(function(result){
			console.log("result.rowsAffected = " + result.rowsAffected);
		});
	}
    
	
	self.searchSong = function(txtSearch){		
		var query = "SELECT *  FROM SongTbl WHERE song_title_ascii LIKE '%" + txtSearch + "%'";
		return Database.query(query).then(function(result){
			return Database.fetchAll(result);
		});
	}
    
    self.searchSongByOffset = function(txtSearch,offset){	
        console.log("offset = " + offset);
		var query = "SELECT *  FROM SongTbl WHERE song_title_ascii LIKE '%" + txtSearch + "%' limit ?,?";
		return Database.query(query,[offset,DB_CONFIG.numItem]).then(function(result){
			return Database.fetchAll(result);
		});
	}
    
    self.searchSongWithRhytmhByOffset = function(rhythm, offset){
        console.log("offset = " + offset);
        var query = "";
        
        if(rhythm != null && rhythm != ""){
		  query = "SELECT *  FROM SongTbl WHERE song_rhythm = '" + rhythm + "' limit ?,?";
        } else {
           query = "SELECT *  FROM SongTbl WHERE song_rhythm IS NULL or song_rhythm = '' limit ?,?"; 
        }
        
		return Database.query(query,[offset,DB_CONFIG.numItem]).then(function(result){
			return Database.fetchAll(result);
		});
    }
    
    self.searchSongByArtist_AuthorByOffset = function(artist_id, offset){	
        console.log("offset = " + offset);
		var query = "SELECT *  FROM SongTbl WHERE song_id IN (SELECT song_id  FROM Song_Author_Tbl WHERE artist_id = ?) limit ?,?";
		return Database.query(query,[artist_id,offset,DB_CONFIG.numItem]).then(function(result){
			return Database.fetchAll(result);
		});
	}
	
    self.searchSongByArtist_SingerByOffset = function(artist_id, offset){	
        console.log("offset = " + offset);
		var query = "SELECT *  FROM SongTbl WHERE song_id IN (SELECT song_id  FROM Song_Singer_Tbl WHERE artist_id = ?) limit ?,?";
		return Database.query(query,[artist_id,offset,DB_CONFIG.numItem]).then(function(result){
			return Database.fetchAll(result);
		});
	}
	
    
    
    self.searchArtist_Author = function(txtSearch){		
		var query = "SELECT ArtistTbl.artist_id , ArtistTbl.artist_name,  COUNT(*) as number   FROM Song_Author_Tbl INNER JOIN ArtistTbl ON Song_Author_Tbl.artist_id = ArtistTbl.artist_id  WHERE Song_Author_Tbl.artist_id IN (SELECT artist_id  FROM ArtistTbl  WHERE artist_ascii LIKE '%" + txtSearch + "%') GROUP BY ArtistTbl.artist_id";
        
		return Database.query(query).then(function(result){
			return Database.fetchAll(result);
		});
	}
    
    self.searchArtist_Singger = function(txtSearch){		
		var query = "SELECT ArtistTbl.artist_id , ArtistTbl.artist_name,  COUNT(*) as number   FROM Song_Singer_Tbl INNER JOIN ArtistTbl ON Song_Singer_Tbl.artist_id = ArtistTbl.artist_id  WHERE Song_Singer_Tbl.artist_id IN (SELECT artist_id  FROM ArtistTbl  WHERE artist_ascii LIKE '%" + txtSearch + "%') GROUP BY ArtistTbl.artist_id";
        
		return Database.query(query).then(function(result){
			return Database.fetchAll(result);
		});
	}
    
    
	return self;

}])

.factory('Utils',[ function(){	
	return {			
        capitalizeFirstLetter : function(string) {
            return string.charAt(0).toUpperCase() + string.slice(1);
        },
        
		setCharAt : function(str,index,chr) {
			if(index > str.length-1) return str;
			return str.substr(0,index) + chr + str.substr(index+1);
		},
		
        getLyric1Lines: function(lyric){
            var lines = lyric.split('\n');	
				
            var linesSplit = [];
            for(var i = 0; i< lines.length; i++){			
                var line = lines[i];
                
                line = line.replace(/\[/g,'<span class="hopamchuan_chord" ng-click="clickChord($event)">[');
			    line = line.replace(/\]/g,']</span>');
                
                linesSplit.push(line);
            }

            //return linesSplit.join('\n'); 
            return linesSplit;
        },
        
         getLyric2Lines: function(lyric){
            var lines = lyric.split('\n');	
				
            var linesSplit = [];
            for(var i = 0; i< lines.length; i++){			
                var line = lines[i];
                var arrLine = this.get2String(line);
                
                linesSplit.push(arrLine[1]);
                linesSplit.push(arrLine[0]);
                //linesSplit.push(line);
            }

            //return linesSplit.join('\n'); 
            return linesSplit; 
        },
        
        getSpaceGroup: function(n){
            var group = "";
            for(var i = 0; i< n; i++){
                group += " ";
            }
            return group;
        },
        
		get2String: function(searchString) {			
			var pattern = /(\[.+?\])/g;
					
			var groupArray = searchString.split(pattern);
            //console.log(groupArray);
            
            var lineMusic = "";
            var lineString = "";
			for(var i = 0; i< groupArray.length; i++){
                if(groupArray[i].match(pattern)){
                   lineMusic += groupArray[i]; 
                } else {
                    lineMusic += this.getSpaceGroup(groupArray[i].length);
                    lineString += groupArray[i] + " ";
                }
            }
			
			
			lineMusic = lineMusic.replace(/\[/g,'<span class="hopamchuan_chord" ng-click="clickChord($event)"> ');
			lineMusic = lineMusic.replace(/\]/g,' </span>');           
            
			return [lineString, lineMusic];
		}
	}
}])

